use alloc_counter::{count_alloc, AllocCounterSystem};
use anyhow::Result;
use std::convert::TryFrom;

#[global_allocator]
static A: AllocCounterSystem = AllocCounterSystem;

static INPUT: &str = include_str!("../../input03");

fn main() -> Result<()> {
    let map = Map::load(INPUT);
    let steps = map.walk(3, 1);
    println!("i walked {} steps.", steps);
    let a = map.walk(1, 1);
    let b = map.walk(3, 1);
    let c = map.walk(5, 1);
    let d = map.walk(7, 1);
    let e = map.walk(1, 2);
    println!("part2 product is {} steps.", a * b * c * d * e);
    Ok(())
}

#[derive(Default)]
struct Map {
    inner: Vec<Vec<Kind>>,
}

impl Map {
    fn load(data: &str) -> Self {
        let (allocs, ret) = count_alloc(|| {
            let mut inner: Vec<Vec<Kind>> = Vec::new();
            data.lines().enumerate().for_each(|(y, line)| {
                let line = line.trim();
                inner.push(vec![Kind::Open; line.len()]);
                line.chars().enumerate().for_each(|(x, c)| {
                    if let Ok(kind) = Kind::try_from(c) {
                        inner[y][x] = kind;
                    }
                })
            });
            Map { inner }
        });
        println!("load allocations: {:?}", allocs);
        ret
    }

    fn walk(&self, x: usize, y: usize) -> usize {
        let (allocs, trees) = count_alloc(|| {
            let mut trees = 0;
            let mut yx = (0, 0);
            loop {
                if yx.0 >= self.inner.len() {
                    break;
                }
                if yx.1 >= self.inner[yx.0].len() {
                    yx.1 -= self.inner[yx.0].len()
                }
                if self.inner[yx.0][yx.1] == Kind::Tree {
                    trees += 1;
                }
                yx.0 += y;
                yx.1 += x;
            }
            trees
        });
        println!("walk allocations: {:?}", allocs);
        trees
    }
}

#[derive(Debug, Clone, Eq, PartialEq)]
enum Kind {
    Open,
    Tree,
}

impl TryFrom<char> for Kind {
    type Error = ();
    fn try_from(c: char) -> Result<Self, ()> {
        match c {
            '.' => Ok(Kind::Open),
            '#' => Ok(Kind::Tree),
            _ => Err(()),
        }
    }
}

#[cfg(test)]
mod tests {
    use super::*;

    #[test]
    fn example() {
        let data = "
..##.......
#...#...#..
.#....#..#.
..#.#...#.#
.#...##..#.
..#.##.....
.#.#.#....#
.#........#
#.##...#...
#...##....#
.#..#...#.#
"
        .trim();
        let map = Map::load(data);
        assert_eq!(map.walk(3, 1), 7)
    }

    #[test]

    fn part2() {
        let data = "
..##.......
#...#...#..
.#....#..#.
..#.#...#.#
.#...##..#.
..#.##.....
.#.#.#....#
.#........#
#.##...#...
#...##....#
.#..#...#.#
"
        .trim();
        let map = Map::load(data);
        let a = map.walk(1, 1);
        let b = map.walk(3, 1);
        let c = map.walk(5, 1);
        let d = map.walk(7, 1);
        let e = map.walk(1, 2);
        assert_eq!(336, a * b * c * d * e)
    }
}
