pub struct Splitter<'a> {
    inner: Option<&'a str>,
}

impl<'a> From<&'a str> for Splitter<'a> {
    fn from(inner: &'a str) -> Self {
        Splitter { inner: Some(inner) }
    }
}

impl<'a> std::ops::Deref for Splitter<'a> {
    type Target = str;
    fn deref(&self) -> &Self::Target {
        match self.inner {
            Some(inner) => inner,
            None => "",
        }
    }
}

impl<'a> Splitter<'a> {
    pub fn get_before(&mut self, r: char) -> Option<&'a str> {
        let inner = self.inner?;
        if let Some(pos) = inner.find(r) {
            let (before, after) = inner.split_at(pos);
            self.inner = Some(after);
            Some(before)
        } else {
            self.inner = None;
            Some(inner)
        }
    }
    pub fn skip(&mut self) {
        self.inner = match self.inner {
            Some(inner) => Some(&inner[1..]),
            None => None,
        };
    }
}
